import random

n = 10 # number of circles
s = 10 # number of starts
rseed = 1 # random number generator seed


def constants(n, s):
    outstr = "\tConstants\n"
    outstr += "\t\tn = {0}\t ! number of circles \n".format(n)
    outstr += "\t\ts = {0}\t ! number of starts \n".format(s)
    outstr += "\tEnd Constants\n"
    return outstr

variables_str = """
    Variables
        x[1:n][1::s] >= -2.0, <= 2.0      ! x-coordinates of circle centers
        y[1:n][1::s] >= -2.0, <= 2.0      ! y-coordinates of circle centers
        circrad[1::s] = 2.0, >= 1.7, <= 2.5   ! radius of circumscribing circle
    End Variables
"""
        
intermediates_str = """
    Intermediates
        ints[1] = 1
        ints[2:n] = ints[1:n-1] + 1
        r[1:n] = ints[1:n]**(-0.5)  ! radii of circles
    End Intermediates
"""
    
def cons1(n,s):
    outstr = "\t\t ! distance between pairs of circles is >= sum of their radii\n"
    for i in range(2,n+1):
        for j in range(1,i):
            outstr += "\t\tsqrt((x[{0}][1::s] - x[{1}][1::s])^2 + (y[{0}][1::s] - y[{1}][1::s])^2) >= r[{0}] + r[{1}]\n".format(i, j)
    return outstr

cons2_str = """
        ! circles are inside circumscribing circle
        circrad[1::s] - sqrt(x[1:n][1::s]^2 + y[1:n][1::s]^2) >= r[1:n]
        
"""

def ranstarts(n,s):
    random.seed(rseed)
    outstr = '\nFile *.csv\n'
    for i in range(1,s+1):
        for j in range(1,n+1):
            outstr += 'packcircles.x[{0}][{1}], {2}\n'.format(j,i,random.uniform(-2,2))
            outstr += 'packcircles.y[{0}][{1}], {2}\n'.format(j,i,random.uniform(-2,2))
    outstr += 'End File'
    return outstr

def gencircleapm(n,s):
    outstr = "Model Pack Circles\n"
    outstr += constants(n,s)
    outstr += variables_str
    outstr += intermediates_str
    outstr += "\n\tEquations\n";
    outstr += cons1(n,s)
    outstr += cons2_str
    outstr += "\t\tminimize circrad[1::s]\n"
    outstr += "\tEnd Equations\nEndModel\n"
    outstr += ranstarts(n,s)
    return outstr

modelstr = gencircleapm(n,s)  # generate apm file
f = open('packcirclesmulti.apm', mode = 'w')
f.write(modelstr)
f.close()


# Import APM Python library apm.py
from apm import *
# Select the server
server = 'http://xps.apmonitor.com'
# Give the application a name
app = 'packcirclesmulti'
# Clear any previous applications by that name
apm(server,app,'clear all')
# Load the model file
apm_load(server,app,'packcirclesmulti.apm')
apm_option(server,app,'nlc.solver',1)
apm_option(server,app,'nlc.coldstart',1)
apm_option(server,app,'nlc.max_iter',500)
# Solve on APM server
solver_output = apm(server,app,'solve')
# Display solver output
print solver_output
# Retrieve results
(results, array) = apm_sol(server,app)
#print results

# get results from python array
cbest = 1000
sbest = 0
for i in range(1,s+1):
    strc = 'packcircles.circrad[{0}]'.format(i)
    cval = array[strc][0]
    print 'result for {0}th start = {1}'.format(i,cval)
    if cval < cbest:
        sbest = i
        cbest = cval

print '\nbest circumscribing radius is {0}\n'.format(cbest)
print 'centers of circles in best packing:'

xs = [] # x coordinates of circle centers
ys = [] # y coordinates of circle centers
for i in range(1,n+1):
    strx = 'packcircles.x[{0}][{1}]'.format(i,sbest)
    stry = 'packcircles.y[{0}][{1}]'.format(i,sbest)
    xval = array[strx][0]
    yval = array[stry][0]
    xs.append(xval)
    ys.append(yval)
    print 'x[{0}] = {1}'.format(i,xval)
    print 'y[{0}] = {1}'.format(i,yval)



# plot results

import pylab
pylab.axes()
cc = pylab.Circle((0,0),cbest,facecolor = 'none')
pylab.gca().add_patch(cc)
for i in range(n):
     cc = pylab.Circle((xs[i],ys[i]),(i+1)**(-0.5),facecolor = 'none')
     pylab.gca().add_patch(cc)
     cntr = pylab.Circle((xs[i],ys[i]),0.01,facecolor = 'b')
     pylab.gca().add_patch(cntr)
     
pylab.axis('scaled')
pylab.show()
